/* 
    OrppCE - Oroo Motioncontroller interface example
    (C) 2005 Jason Hunt

    nulluser@gmail.com

    FILE: main.cpp

    Note: had to change the resource include to "rewres.h"

*/


#include <windows.h>			// Windows core
#include <commctrl.h>
#include <aygshell.h>			// Shell interface

#include "main.h"
#include "resource.h"
#include "display.h"

#include "orppce.h"

orpp_class orpp("COM1");		// The orpp device


// Saved windows handles
HWND main_win = NULL;	
HWND system_hwnd = NULL;
HWND main_menu = NULL;

HWND input_dialog = NULL;
HWND output_dialog = NULL;
HWND servo_dialog = NULL;


HINSTANCE hinst;	


/*****************
* User functions *
*****************/


/* Called on program start */
void program_start(void)
{
	add_line("OrppCE (C) 2005 Jason Hunt");
	add_line("System Start - Version "VERSION);    
	
	// Link the all_line funciton to the orpp interface
	orpp.set_error_function(add_line);
	orpp.set_message_function(add_line);

	// Take a sample 10 times per second
    SetTimer(system_hwnd, ID_SAMPLE, 100, NULL);
}
/* End of program start /


/* Called on program stop */
void program_end ( void)
{
    
	// Destory dialogs
	DestroyWindow(input_dialog);	
    DestroyWindow(output_dialog);	    	
	DestroyWindow(servo_dialog);	
	
	// Destory main window to complete shutdown
	DestroyWindow(main_win);
}
/* End of program end */


/************************
* End of user functions *
************************/



/*******************************
* Window and dialog procedures *
*******************************/


/* Deal with the system timer */
void timer_message(WPARAM wParam)
{
	static int count = 0;
    
	
	if (wParam == ID_SAMPLE) // Do sample?
    {    
		// Check for reconnect
		if (!orpp.get_comm_ok() && count++ > 10)
		{
			count = 0;
			orpp.connect();
			return;
		}

		// Get all inputs
		orpp.get_all();
		
		// Update digial inputs in dialog
		SetDlgItemInt(input_dialog, IDC_DIGITALIN0, orpp.get_last(orpp_digital_in, 0), FALSE);
	    SetDlgItemInt(input_dialog, IDC_DIGITALIN1, orpp.get_last(orpp_digital_in, 1), FALSE);
	    SetDlgItemInt(input_dialog, IDC_DIGITALIN2, orpp.get_last(orpp_digital_in, 2), FALSE);
	    SetDlgItemInt(input_dialog, IDC_DIGITALIN3, orpp.get_last(orpp_digital_in, 3), FALSE);
	    SetDlgItemInt(input_dialog, IDC_DIGITALIN4, orpp.get_last(orpp_digital_in, 4), FALSE);

        // Update digital outpus in dialog 
		SetDlgItemInt(input_dialog, IDC_ANALOGIN0, orpp.get_last(orpp_analog, 0), FALSE);
        SetDlgItemInt(input_dialog, IDC_ANALOGIN1, orpp.get_last(orpp_analog, 1), FALSE);
        SetDlgItemInt(input_dialog, IDC_ANALOGIN2, orpp.get_last(orpp_analog, 2), FALSE);
        SetDlgItemInt(input_dialog, IDC_ANALOGIN3, orpp.get_last(orpp_analog, 3), FALSE);
        SetDlgItemInt(input_dialog, IDC_ANALOGIN4, orpp.get_last(orpp_analog, 4), FALSE);
	}   
}   
/* End of timer message */


/* Create the menu */
void menu_init( HWND hwnd )
{
    SHMENUBARINFO mbi;

    //set up menu bar structure
    memset(&mbi, 0, sizeof(SHMENUBARINFO));
    mbi.cbSize     = sizeof(SHMENUBARINFO);
    mbi.hwndParent = hwnd;
    mbi.nToolBarId = IDM_MAINMENU;
    mbi.hInstRes   = hinst;
    mbi.nBmpId     = 0;
    mbi.cBmpImages = 0;	
        
    main_menu = mbi.hwndMB;

    SHCreateMenuBar(&mbi);
}
/* End of menu_init */


/* Windows procedure */
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) 
	{
//      case WM_CHAR: process_char(wParam & 0xff); break;
//	    case WM_MOUSEMOVE:    break;
//	    case WM_LBUTTONDOWN:   break;

	
	    case WM_TIMER: timer_message(wParam); break;

		case WM_PAINT: display_text(); break;

			
	    case WM_DESTROY:  		    
			DestroyWindow(main_menu);
			PostQuitMessage (0);
            break;

		case WM_COMMAND:	// Menu command

		    switch (LOWORD(wParam)) // decode menu
			{
			    case IDM_EXIT:
				    program_end();
					break;

				case IDM_INPUT:							
					ShowWindow(servo_dialog, SW_HIDE);		// Hide all dialogs	
					UpdateWindow(servo_dialog);				    					
					
					ShowWindow(output_dialog, SW_HIDE);
					UpdateWindow(output_dialog);
				
   				    ShowWindow(input_dialog, SW_SHOW);		// Display dialog
					UpdateWindow(input_dialog);
					break;
				
				case IDM_OUTPUT:
					ShowWindow(servo_dialog, SW_HIDE);
					UpdateWindow(servo_dialog);				    
					
					ShowWindow(input_dialog, SW_HIDE);
					UpdateWindow(input_dialog);

					ShowWindow(output_dialog, SW_SHOW);
					UpdateWindow(output_dialog);					
					break;
				
				case IDM_SERVO:
					ShowWindow(output_dialog, SW_HIDE);
					UpdateWindow(output_dialog);				    

					ShowWindow(input_dialog, SW_HIDE);
				    UpdateWindow(input_dialog);

					ShowWindow(servo_dialog ,SW_SHOW);
					UpdateWindow(servo_dialog);					
					break;
			} break;

		// Create menu bar and go full screen 		
		case WM_CREATE: 
			 menu_init(hWnd);
			 system_hwnd = hWnd;			 
		 	break;

        default:
          return DefWindowProc(hWnd, message, wParam, lParam);
	}
    return 0;
}

/* End of windproc */



/* Callback proc for the input form */
LRESULT CALLBACK input_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) 
	{
        case WM_PAINT: 
		{

		}

		break;
	}

	return(0);
}
/* End of input_proc */


/* Callback proc for the output form */
LRESULT CALLBACK output_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static HWND trackbar[2];

    switch (message) 
	{
		case WM_INITDIALOG:          
		{
			// Get tracker handles
			trackbar[0]	= GetDlgItem(hWnd, IDC_PWM_0);
			trackbar[1]	= GetDlgItem(hWnd, IDC_PWM_1);

			// Set range to 0..255
            SendMessage(trackbar[0], TBM_SETRANGE,  true, MAKELONG (0, 255));
            SendMessage(trackbar[1], TBM_SETRANGE,  true, MAKELONG (0, 255));
          
		} break;
	
		case WM_HSCROLL:
		{
            switch LOWORD(wParam) 
			{
				// Update on these conditions
				case TB_BOTTOM:	
				case SB_THUMBTRACK:
                case TB_THUMBPOSITION:
                case TB_LINEUP:
                case TB_LINEDOWN:
                case TB_PAGEUP:
                case TB_PAGEDOWN:
                case TB_TOP:

				// get trackbar handle
				HWND t = (HWND)(lParam);

			    // get position
				int pos = SendMessage(t, TBM_GETPOS, 0, 0);			

				// Decode trackbar handle
				if (t == trackbar[0]) { orpp.set(orpp_pwm, 0, pos);	SetDlgItemInt(hWnd, IDC_PWM_VAL0, pos, FALSE); }
				if (t == trackbar[1]) { orpp.set(orpp_pwm, 1, pos);	SetDlgItemInt(hWnd, IDC_PWM_VAL1, pos, FALSE); }
			} 					

		} break;
		case WM_COMMAND:	// Decode buttons
		{
			switch(wParam) 
			{		
				case IDC_DIGITAL0:orpp.toggle(orpp_digital_out, 0); break;
				case IDC_DIGITAL1:orpp.toggle(orpp_digital_out, 1); break;
				case IDC_DIGITAL2:orpp.toggle(orpp_digital_out, 2); break;
				case IDC_DIGITAL3:orpp.toggle(orpp_digital_out, 3); break;
				case IDC_DIGITAL4:orpp.toggle(orpp_digital_out, 4); break;
				case IDC_DIGITAL5:orpp.toggle(orpp_digital_out, 5); break;
				
				case IDC_OC0: orpp.toggle(orpp_opencollector, 0); break;
				case IDC_OC1: orpp.toggle(orpp_opencollector, 1); break;

				case IDC_LED0: orpp.toggle(orpp_led, 0); break;
				case IDC_LED1: orpp.toggle(orpp_led, 1); break;				
			} 
		} break;

	}    
	return(0);
}
/* End of output_proc */


/* Callback proc for the servo form */
LRESULT CALLBACK servo_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static HWND trackbar[6];	
	
	switch (message) 
	{
		case WM_INITDIALOG:          
		{
			// Get tracker handles
			trackbar[0]	= GetDlgItem(hWnd, IDC_SERVO_0);
			trackbar[1]	= GetDlgItem(hWnd, IDC_SERVO_1);
			trackbar[2]	= GetDlgItem(hWnd, IDC_SERVO_2);
    		trackbar[3]	= GetDlgItem(hWnd, IDC_SERVO_3);
			trackbar[4]	= GetDlgItem(hWnd, IDC_SERVO_4);
			trackbar[5]	= GetDlgItem(hWnd, IDC_SERVO_5);

			// Set range to 0..255
            SendMessage(trackbar[0], TBM_SETRANGE,  true, MAKELONG (0, 255));
            SendMessage(trackbar[1], TBM_SETRANGE,  true, MAKELONG (0, 255));
            SendMessage(trackbar[2], TBM_SETRANGE,  true, MAKELONG (0, 255));
            SendMessage(trackbar[3], TBM_SETRANGE,  true, MAKELONG (0, 255));
            SendMessage(trackbar[4], TBM_SETRANGE,  true, MAKELONG (0, 255));
            SendMessage(trackbar[5], TBM_SETRANGE,  true, MAKELONG (0, 255));

			// Set to center position
            SendMessage(trackbar[0], TBM_SETPOS,  true, 128);
            SendMessage(trackbar[1], TBM_SETPOS,  true, 128);
            SendMessage(trackbar[2], TBM_SETPOS,  true, 128);
            SendMessage(trackbar[3], TBM_SETPOS,  true, 128);
            SendMessage(trackbar[4], TBM_SETPOS,  true, 128);
            SendMessage(trackbar[5], TBM_SETPOS,  true, 128);

		} break;
	
	
		case WM_PAINT: 
		{

		} break;

		case WM_HSCROLL:
		{
            switch LOWORD(wParam) 
			{
				// Update on these conditions
				case TB_BOTTOM:
                case TB_THUMBPOSITION:
				case SB_THUMBTRACK:
                case TB_LINEUP:
                case TB_LINEDOWN:
                case TB_PAGEUP:
                case TB_PAGEDOWN:
                case TB_TOP:

				// Get trackbar handle
				HWND t = (HWND)(lParam);

			    // Get position
				int pos = SendMessage(t, TBM_GETPOS, 0, 0);			

				// Decode trackbar handle
				if (t == trackbar[0]) { orpp.set(orpp_servo, 0, pos);	SetDlgItemInt(hWnd, IDC_SERVO_VAL1, pos, FALSE); }
				if (t == trackbar[1]) { orpp.set(orpp_servo, 1, pos);	SetDlgItemInt(hWnd, IDC_SERVO_VAL2, pos, FALSE); }
				if (t == trackbar[2]) { orpp.set(orpp_servo, 2, pos);	SetDlgItemInt(hWnd, IDC_SERVO_VAL3, pos, FALSE); }
				if (t == trackbar[3]) { orpp.set(orpp_servo, 3, pos);	SetDlgItemInt(hWnd, IDC_SERVO_VAL4, pos, FALSE); }
				if (t == trackbar[4]) { orpp.set(orpp_servo, 4, pos);	SetDlgItemInt(hWnd, IDC_SERVO_VAL5, pos, FALSE); }
				if (t == trackbar[5]) { orpp.set(orpp_servo, 5, pos);	SetDlgItemInt(hWnd, IDC_SERVO_VAL6, pos, FALSE); }
			} 
		} break;
	}
	return(0);
}



/**************************************
* End of window and dialog procedures *
**************************************/


/* Window Main */
int WINAPI WinMain(	HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPTSTR    lpCmdLine, int       nCmdShow)
{
    static const TCHAR	window_title[] = TEXT("OrppCE"); 	// Title and class
    static const TCHAR	window_class[] = TEXT("OrppCE");

	
	// If it's running, just activate it
    if (FindWindow(window_class, window_title))
	{
        SetForegroundWindow(FindWindow(window_class, window_title));
        return(0);
	}    
		
	hinst =  hInstance;		// Instance is need later on

	// Create the window
	WNDCLASS wc;
  
    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= window_class;
  
    RegisterClass(&wc);


    main_win = CreateWindow(window_class, window_title, WS_VISIBLE ,
                            CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
						    CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
  
    if (!main_win) return FALSE;


	RECT rc;

    // Enter full screen mode
    GetWindowRect(main_win, &rc);

    SHFullScreen(main_win, SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON);
      	  
	// Adjust for removed taskbar
    MoveWindow(main_win,  rc.left, rc.top-MENU_HEIGHT, rc.right, rc.bottom, TRUE);
 				
    ShowWindow(main_win, nCmdShow);
	UpdateWindow(main_win);
	 
	// Setup dialog boxes
	input_dialog = CreateDialog(hinst, MAKEINTRESOURCE(IDD_INPUT),
						        main_win, (DLGPROC)input_proc); 
        	
	output_dialog = CreateDialog(hinst, MAKEINTRESOURCE(IDD_OUTPUT),
						         main_win, (DLGPROC)output_proc); 

	servo_dialog = CreateDialog(hinst, MAKEINTRESOURCE(IDD_SERVO),
						        main_win, (DLGPROC)servo_proc); 

    ShowWindow(input_dialog , SW_SHOW);
    UpdateWindow(input_dialog );

    setup_text();					// Clear console buffer

	program_start();				// User start
      	
	MSG m;

	// The windows message loop
    while (GetMessage(&m, NULL, 0, 0)) 
	{
        TranslateMessage(&m);
        DispatchMessage(&m);
	}
  
    return(m.wParam);
}
/* End of WinMain */
